package com.motor.web.controller.motor;



import com.motor.common.core.domain.AjaxResult;
import com.motor.system.domain.req.LanEquipmentCallbackReq;
import com.motor.system.domain.req.TimeSynCallbackReq;
import com.motor.system.domain.vo.LanEquipmentVo;
import com.motor.system.domain.vo.LogPropertyVo;
import com.motor.system.service.IStationMachineService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.compress.utils.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@RestController
@RequestMapping("/station/machine")
public class StationMachineController {

    @Autowired
    private IStationMachineService stationMachineService;

    @Value("${material.server}")
    private String materialServer;

    @Value("${swagger.pathMapping}")
    private String pathMapping;

    /**
     * 局域网设备信息_回调接口，提供给安全开发用
     * 局域网设备信息调出（地磅、刹车等）（Mac，IP，设备名称）
     * @param req
     * @return
     */
    @PostMapping("/lan_equipment_callback")
    public String lanEquipmentCallback(@RequestBody LanEquipmentCallbackReq req) {
        String callback = stationMachineService.lanEquipmentCallback(req);
        return callback;
    }


    /**
     * 前端按钮调用展示接口
     * 局域网设备信息调出（地磅、刹车等）（Mac，IP，设备名称）

     {
     "192.168.100.0": [
     {
     "host": " 192.168.100.1",
     "mac": " MAC Address Unknown",
     "name": "DESKTOP-NEUAD24"
     },
     {
     "host": " 192.168.100.13",
     "mac": " 00:0C:29:AB:D1:10",
     "name": "192.168.100.13"
     },
     {
     "host": " 192.168.100.20",
     "mac": " 00:50:56:EB:BF:17",
     "name": "192.168.100.20"
     }
     ]
     }

     * @return
     */
    @GetMapping("/lan_equipment")
    public List<LanEquipmentVo> lanEquipment() {
        List<LanEquipmentVo> map = Lists.newArrayList();

        LanEquipmentVo lanEquipmentVo = new LanEquipmentVo();
        lanEquipmentVo.setIp("1.1.1.1");
        lanEquipmentVo.setHost("192.168.100.20");
        lanEquipmentVo.setMac("00:50:56:EB:BF:17");
        lanEquipmentVo.setName("192.168.100.20");
        map.add(lanEquipmentVo);

        LanEquipmentVo lanEquipmentVo3 = new LanEquipmentVo();
        lanEquipmentVo3.setIp("1.1.1.1");
        lanEquipmentVo3.setHost("192.168.100.20");
        lanEquipmentVo3.setMac("00:50:56:EB:BF:17");
        lanEquipmentVo3.setName("192.168.100.20");
        map.add(lanEquipmentVo3);

        LanEquipmentVo lanEquipmentVo2 = new LanEquipmentVo();
        lanEquipmentVo2.setIp("2.2.2.2");
        lanEquipmentVo2.setHost("192.168.100.13");
        lanEquipmentVo2.setMac("00:50:56:EB:BF:17");
        lanEquipmentVo2.setName("192.168.100.20");
        map.add(lanEquipmentVo2);

        LanEquipmentVo lanEquipmentVo4 = new LanEquipmentVo();
        lanEquipmentVo4.setIp("2.2.2.2");
        lanEquipmentVo4.setHost("192.168.100.13");
        lanEquipmentVo4.setMac("00:50:56:EB:BF:17");
        lanEquipmentVo4.setName("192.168.100.20");
        map.add(lanEquipmentVo4);

       // List<LanEquipmentVo> map = stationMachineService.lanEquipment();
        return map;
    }


    /**
     * exe文件上传
     */
    @PostMapping("/upload_exe")
    public AjaxResult uploadExe(@RequestParam("file") MultipartFile file) throws Exception {
        AjaxResult result = stationMachineService.uploadExe(file);
        return result;
    }

    /**
     * 版本一致性
     *
     * 当前版本为：以下为返回结果
     * {
     *   "version": "1.4.1.1022"
     * }
     *
     * @param consistency
     * @return
     */
    @GetMapping("/consistency")
    public String consistency(@RequestParam("consistency") String consistency) {
        //String callback = stationMachineService.consistency(consistency);
        String callback = "1.4.1.1022";
        return callback;
    }

    /**
     * 上传后获取搜索项
     * @return
     */
    @GetMapping("/selected")
    public Set<String> selected() {
        Set<String> sets = Sets.newHashSet();
        sets.add("everything.exe");
        sets.add("hmark.exe");
        sets.add("nmap-7.95-setup.exe");
        //Set<String> sets = stationMachineService.selected();
        return sets;
    }

    /**
     * 历史日志及日志管理-弹出搜索框
     * @return
     */
    @GetMapping("/fileSearchEngine")
    public String fileSearchEngine() {
        //String engine = stationMachineService.fileSearchEngine();
        String engine = "True";
        return engine;
    }

    /**
     * 历史日志及日志管理

     {
     "logFileContinueTime": "文件保留时长: 3:48:38.040051",
     "logFileCreateDate": "文件首次建立日期: 2024-05-18 13:23:17.427685",
     "logFileLastModifyDate": "文件最后修改日期: 2024-05-18 14:52:43.758790",
     "logFilePrivilege": "发现注入Flag: \"KeywareTestFlag_LogChange 2024-05-18 18:41:21\" 于 D:\\文件_lxy\\项目\\内蒙车检产品\\MET_auto\\data\\upload\\log_upload\\log.log 文件的第 224 行可查看该证明文件可修改"
     }

     * @param logFilePath
     * @return
     */
    @GetMapping("/logProperty")
    public LogPropertyVo logProperty(@RequestParam("logFilePath") String logFilePath) {
        LogPropertyVo property = stationMachineService.logProperty(logFilePath);
        //测试
//        LogPropertyVo property = new LogPropertyVo();
//        property.setLogFileContinueTime("文件保留时长: 3:48:38.040051");
//        property.setLogFileCreateDate("文件首次建立日期: 2024-05-18 13:23:17.427685");
//        property.setLogFileLastModifyDate("文件最后修改日期: 2024-05-18 14:52:43.758790");
//        property.setLogFilePrivilege("发现注入Flag: \\\"KeywareTestFlag_LogChange 2024-05-18 18:41:21\\\" 于 D:\\\\文件_lxy\\\\项目\\\\内蒙车检产品\\\\MET_auto\\\\data\\\\upload\\\\log_upload\\\\log.log 文件的第 224 行可查看该证明文件可修改");
        return property;
    }


    /**
     * 时间同步性-安全回调
     * 未发现时间同步服务器
     * @return
     */
    @PostMapping("/time_syn_callback")
    public String timeSynCallback(@RequestBody TimeSynCallbackReq req) {
        String timeSynCallback = stationMachineService.timeSynCallback(req);

        return timeSynCallback;
    }

    /**
     * 时间同步性-查询
     * @return
     */
    @GetMapping("/query_time_syn")
    public Map<String,List<StringBuilder>> queryTimeSyn() {
        Map<String, List<StringBuilder>> timeSyn = stationMachineService.queryTimeSyn();
//        Map<String, List<StringBuilder>> timeSyn = new HashMap<>();
//        StringBuilder append = new StringBuilder().append("正在跟踪 192.168.100.15 [192.168.100.15:123]。\\n正在收集 1 示例。\\n当前时间是 2024/5/18 下午 4:27:59。\\n16:27:59, -00.3113682s\\n");
//        List<StringBuilder>  list = new ArrayList<StringBuilder>();
//        list.add(append);
//
//        StringBuilder append1 = new StringBuilder().append("正在跟踪 192.168.100.15 [192.168.100.15:123]。\\n正在收集 1 示例。\\n当前时间是 2024/5/18 下午 4:27:59。\\n16:27:59, -00.3113682s\\nFSDSF\\nFFSDFSFSFSDF\\nFDSFSSDF\\n");
//        List<StringBuilder>  list1 = new ArrayList<StringBuilder>();
//        list1.add(append1);
//
//        timeSyn.put("192.168.100.15",list);
//        timeSyn.put("192.168.100.20",list1);
        return timeSyn;
    }


    /**
     *局域网设备信息-网络括扑图-触发安全接口
     * @return
     */
    @GetMapping("/equipment_message_job")
    public String equipmentMessageCallbackJob() {
        String equipmentMessageCallbackJob = stationMachineService.equipmentMessageCallbackJob();
        return equipmentMessageCallbackJob;
    }

    /**
     * 时间同步性，触发安全接口
     * @return
     */
    @GetMapping("/query_time_syn_job")
    public String queryTimeSynJob() {
        String timeSynCallback = stationMachineService.queryTimeSynJob();
        return timeSynCallback;
    }


}
