package com.motor.system.service.impl;


import com.alibaba.fastjson2.JSON;
import com.motor.common.core.domain.AjaxResult;
import com.motor.system.domain.req.LanEquipmentCallbackReq;
import com.motor.system.domain.req.TimeSynCallbackReq;
import com.motor.system.domain.vo.ConsistencyResp;
import com.motor.system.domain.vo.LanEquipmentVo;
import com.motor.system.domain.vo.LogPropertyVo;
import com.motor.system.service.IStationMachineService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.compress.utils.Sets;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

@Slf4j
@Service
public class StationMachineServiceImpl implements IStationMachineService {

    //TODO  局域网设备信息,暂时定义全局变量，后续考虑存入数据库
    private static List<String> list= Lists.newArrayList();

    /**
     * 局域网设备信息
     */
    private static List<LanEquipmentVo> mapList = Lists.newArrayList();

    /**
     * 时间同步性信息
     */
    private static Map<String, List<StringBuilder>> timeSynMapList = new HashMap<>();

    private static Set<String> selectedSets = Sets.newHashSet();

    private static String flag = null;

    @Value("${material.server}")
    private String materialServer;

    @Value("${material.exeUrl}")
    private String materialExeUrl;

    @Value("${material.logUrl}")
    private String materialLogUrl;

    @Value("${swagger.pathMapping}")
    private String pathMapping;





    @Override
    public List<LanEquipmentVo> lanEquipment() {
        return mapList;
    }

    /**
     * 局域网设备信息_回调接口，提供给安全开发用
     * @param req
     * @return
     */
    @Override
    public String lanEquipmentCallback(LanEquipmentCallbackReq req) {
        System.out.println("req:"+req.toString());
        List<Map<String, String>> listMap = req.getResultFile();
        if (CollectionUtils.isEmpty(listMap)) {
            return null;
        }
        mapList.clear();
       // String str = "E:\\呼和浩特市\\24_result.txt";
        //D:/文件_lxy/项目/内蒙车检产品/scriptTest/result/nmapResult/192.168.100.0/24_result.txt
        for (Map<String, String> maps : listMap) {
            Set<Map.Entry<String, String>> entries = maps.entrySet();
            for (Map.Entry<String, String> entry : entries) {
                String replacedUrl = entry.getValue().replace("/", "\\");
                List<LanEquipmentVo> txtFile = readTxtFile(replacedUrl,entry.getKey());
                mapList.addAll(txtFile);
            }
        }
        return null;
    }

    // 读取文件内容  参数要完成路径和文件名 String filePathName="D:/test/tgj/test1.txt";

    /**
     * 解析局域网设备信息
     * @param filePathName
     * @return
     */
    private List<LanEquipmentVo> readTxtFile(String filePathName,String ip){
        List<LanEquipmentVo> strList = new ArrayList<>();
        BufferedReader reader;
        try {
            reader = new BufferedReader(new FileReader(filePathName));
            String line = reader.readLine();
            while (line != null) {
                if ("----------".equals(line)){
                    line = reader.readLine();// 继续读取下一行
                    continue;
                }
                LanEquipmentVo lanEquipmentVo = new LanEquipmentVo();
                int end = line.indexOf("(")-1;
                lanEquipmentVo.setIp(ip);
                lanEquipmentVo.setHost(line.substring(5,end));
                lanEquipmentVo.setName(line.substring(line.indexOf("(")+1,line.length()-1));
                line = reader.readLine();// 继续读取下一行
                lanEquipmentVo.setMac(line.substring(4));
                strList.add(lanEquipmentVo);
                line = reader.readLine();// 继续读取下一行
            }
            reader.close();
            return strList;
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("文件不存在");
            return strList;
        }
    }


    @Override
    public String consistency(String consistency) {
        // 创建HttpClient实例
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            // 创建HttpPost实例，设置请求的URL
            //HttpPost httpPost = new HttpPost("http://192.168.0.3:9999/exeVersion");
            HttpPost httpPost = new HttpPost("http://127.0.0.1:9999/exeVersion");

            // 设置请求体（例如JSON数据）
            String json = "{\"exeName\":\"" + consistency + "\"}";
            StringEntity entity = new StringEntity(json,"utf-8");
            entity.setContentType("application/json");
            httpPost.setEntity(entity);

            // 执行请求并获得响应
            HttpResponse response = httpClient.execute(httpPost);

            // 从响应中提取数据
            String result = EntityUtils.toString(response.getEntity());
            System.out.println("Response: " + result);
            ConsistencyResp object = JSON.parseObject(result, ConsistencyResp.class);
            return object.getVersion();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                httpClient.close(); // 关闭HttpClient实例
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Override
    public AjaxResult uploadExe(MultipartFile file) throws Exception {
        AjaxResult ajax = AjaxResult.success();
        try {
            log.info("***************upload file start***************");
            if (!file.isEmpty()) {
                long fileSize = file.getSize();
                if(fileSize > 2097152000L) {
                    return AjaxResult.error("上传失败，文件大小不能超过1.8GB。");
                }
                //快捷方式文件通常比正常文件要小
                if (file.getSize() < 1024) {
                    return AjaxResult.error("上传失败，请上传正确的文件类型。");
                }
                String fileName = file.getOriginalFilename();
                String extName = fileName.substring(fileName.lastIndexOf(".") + 1).toLowerCase();
                List<String> fileExt = new ArrayList<String>();
                fileExt.add("exe");
                if(!fileExt.contains(extName)) {
                    return AjaxResult.error("上传的不是exe扩展名文件，请重新上传。");
                }
                // 获取当前用户目录，即程序运行的盘符
                String currentDir = System.getProperty("user.dir");
                // 输出当前用户目录，即盘符
                System.out.println("当前运行程序的盘符: " + currentDir);
                String drive = currentDir.substring(0, 1);
                String exeUrl =  drive + materialExeUrl;
                String logUrl =  drive + materialLogUrl;


                File downloadFile = new File(exeUrl);
                //如果不存在下载目录，进行创建
                if(!downloadFile.exists()) {
                    log.info("downloadActualForecastRainfall downloadPath not exits, mkdir this dir {}", exeUrl);
                    downloadFile.mkdirs();
                }
                // 获取文件的字节
                byte[] bytes = file.getBytes();
                // 使用文件字节创建文件
                // 注意：需要提供保存文件的路径，这里假设是在项目根目录下的exeUrl文件夹
                Path path = Paths.get(exeUrl + fileName);
                // 写入文件
                Files.write(path, bytes);

                ajax.put("filePath", exeUrl);
                Set<String> sets = Sets.newHashSet();
                // 或者更安全的做法是先检查再添加
                if (!sets.contains(fileName)) {
                    sets.add(fileName);
                }
                if (!selectedSets.contains(fileName)) {
                    selectedSets.add(fileName);
                }
                System.out.println("sets--------------------"+sets);
                ajax.put("selected",sets);
                log.info("***************upload file end***************");
            }
        } catch (Exception e) {
            log.error("error", e.fillInStackTrace());
            throw e;
        }
        return ajax;
    }

    @Override
    public String fileSearchEngine() {
        try {
            // 远程Python接口的URL；http://<IP地址>:<端口>/<接口路径>
            //String url = "http://192.168.0.3:9999/fileSearchEngine";
            String url = "http://127.0.0.1:9999/fileSearchEngine";

            // 创建HttpClient实例
            HttpClient httpClient = HttpClients.createDefault();
            // 创建HttpGet请求对象
            HttpGet httpGet = new HttpGet(url);
            // 执行请求并获得响应
            HttpResponse response = httpClient.execute(httpGet);
            // 从响应中提取内容
            String responseContent = EntityUtils.toString(response.getEntity());
            // 输出响应内容
            System.out.println(responseContent);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Set<String> selected() {
        return selectedSets;
    }

    @Override
    public LogPropertyVo logProperty(String logFilePath) {
        // 创建HttpClient实例
        CloseableHttpClient httpClient = HttpClients.createDefault();
        try {
            // 创建HttpPost实例，设置请求的URL
            //HttpPost httpPost = new HttpPost("http://192.168.0.3:9999/logProperty");
            HttpPost httpPost = new HttpPost("http://127.0.0.1:9999/logProperty");

            // 设置请求体（例如JSON数据）
            String json = "{\"logFile_path\":\"" + logFilePath + "\"}";
            StringEntity entity = new StringEntity(json,"utf-8");
            entity.setContentType("application/json");
            httpPost.setEntity(entity);

            // 执行请求并获得响应
            HttpResponse response = httpClient.execute(httpPost);

            // 从响应中提取数据
            String result = EntityUtils.toString(response.getEntity());

            LogPropertyVo object = JSON.parseObject(result, LogPropertyVo.class);

            return object;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                httpClient.close(); // 关闭HttpClient实例
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 时间同步性-安全回调
     * @param req
     * @return
     */
    @Override
    public String timeSynCallback(TimeSynCallbackReq req) {
        System.out.println("req:"+req.toString());
        log.info("时间同步性-安全回调:{}", ObjectUtils.isEmpty(req)?"":req.toString());
        timeSynMapList.clear();
        List<Map<String, String>> listMap = req.getTimeSynResult();
        if (CollectionUtils.isEmpty(listMap)) {
            return null;
        }
        //String str = "E:\\呼和浩特市\\timeDiff_192.168.100.15.txt";
        //D:\\文件_lxy\\项目\\内蒙车检产品\\MET_auto\\script\\..\\result\\timeDiff_192.168.100.15.txt
        for (Map<String, String> maps : listMap) {
            Set<Map.Entry<String, String>> entries = maps.entrySet();
            for (Map.Entry<String, String> entry : entries) {
                String replacedUrl = entry.getValue().replace("\\", "\\");
                List<StringBuilder> timeSynVoList = readTimeSynTxtFile(replacedUrl);
                timeSynMapList.put(entry.getKey(),timeSynVoList);
            }
        }
        return "";
    }

    /**
     * 解析局域网设备信息
     * @param filePathName
     * @return
     */
    private List<StringBuilder> readTimeSynTxtFile(String filePathName){
        List<StringBuilder> strList = new ArrayList<>();
        BufferedReader reader;
        try {
            StringBuilder content = new StringBuilder();
            InputStreamReader isr = new InputStreamReader(new FileInputStream(filePathName), "UTF-8");
            reader = new BufferedReader(isr);
            String line = null;
            while ((line = reader.readLine()) != null) {
                content.append(line);
                content.append("\n");
            }
            strList.add(content);
            reader.close();
            return strList;
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("文件不存在");
            return strList;
        }
    }

    @Override
    public Map<String, List<StringBuilder>> queryTimeSyn() {
        return timeSynMapList;
    }



    @Override
    public String equipmentMessageCallbackJob() {
        try {
            // 远程Python接口的URL；http://<IP地址>:<端口>/<接口路径>
            //String url = "http://192.168.0.3:9999/ipDetect";
            String url = "http://127.0.0.1:9999/ipDetect";


            // 创建HttpClient实例
            HttpClient httpClient = HttpClients.createDefault();
            // 创建HttpGet请求对象
            HttpGet httpGet = new HttpGet(url);
            // 执行请求并获得响应
            HttpResponse response = httpClient.execute(httpGet);
            // 从响应中提取内容
            String responseContent = EntityUtils.toString(response.getEntity());
            // 输出响应内容
            System.out.println(responseContent);
            return responseContent;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }


    @Override
    public String queryTimeSynJob() {
        log.info("时间同步性定时任务开始");
        System.out.println();
        try {
                // 远程Python接口的URL；http://<IP地址>:<端口>/<接口路径>
                //String url = "http://192.168.0.3:9999/timeSyn";
                String url = "http://127.0.0.1:9999/timeSyn";

                // 创建HttpClient实例
                HttpClient httpClient = HttpClients.createDefault();
                // 创建HttpGet请求对象
                HttpGet httpGet = new HttpGet(url);
                // 执行请求并获得响应
                HttpResponse response = httpClient.execute(httpGet);
                // 从响应中提取内容
                String responseContent = EntityUtils.toString(response.getEntity());
                // 输出响应内容
                System.out.println(responseContent);
                return responseContent;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }



}
